"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const every = require("lodash/every.js");
const some = require("lodash/some.js");
const startsWith = require("lodash/startsWith.js");
const endsWith = require("lodash/endsWith.js");
const lt = require("lodash/lt.js");
const lte = require("lodash/lte.js");
const gt = require("lodash/gt.js");
const gte = require("lodash/gte.js");
const eq = require("lodash/eq.js");
const map = require("lodash/map.js");
const keyBy = require("lodash/keyBy.js");
const chunk = require("lodash/chunk.js");
const drop = require("lodash/drop.js");
const dropRight = require("lodash/dropRight.js");
const take = require("lodash/take.js");
const takeRight = require("lodash/takeRight.js");
const flattenDepth = require("lodash/flattenDepth.js");
const fromPairs = require("lodash/fromPairs.js");
const nth = require("lodash/nth.js");
const reverse = require("lodash/reverse.js");
const uniq = require("lodash/uniq.js");
const uniqBy = require("lodash/uniqBy.js");
const countBy = require("lodash/countBy.js");
const filter = require("lodash/filter.js");
const reject = require("lodash/reject.js");
const groupBy = require("lodash/groupBy.js");
const sortBy = require("lodash/sortBy.js");
const minBy = require("lodash/minBy.js");
const maxBy = require("lodash/maxBy.js");
const meanBy = require("lodash/meanBy.js");
const sumBy = require("lodash/sumBy.js");
const join = require("lodash/join.js");
const get = require("lodash/get.js");
const set = require("lodash/set.js");
const unset = require("lodash/unset.js");
const assign = require("lodash/assign.js");
const mapValues = require("lodash/mapValues.js");
const at = require("lodash/at.js");
const toPairs = require("lodash/toPairs.js");
const invert = require("lodash/invert.js");
const invertBy = require("lodash/invertBy.js");
const keys = require("lodash/keys.js");
const values = require("lodash/values.js");
const omit = require("lodash/omit.js");
const transformations = {
    array: {
        each: (array, arg) => {
            return map(array, item => applyTransformations(item, arg));
        },
        map,
        keyBy,
        chunk,
        drop,
        dropRight,
        take,
        takeRight,
        flattenDepth,
        fromPairs,
        nth,
        reverse,
        uniq,
        uniqBy,
        countBy,
        filter,
        reject,
        filterIf: (array, arg) => {
            return filter(array, item => applyTransformations(item, arg));
        },
        rejectIf: (array, arg) => {
            return reject(array, item => applyTransformations(item, arg));
        },
        groupBy,
        sortBy,
        minBy,
        maxBy,
        meanBy,
        sumBy,
        join,
    },
    object: {
        get,
        assign: (src, args) => args.reduce((obj, path) => {
            const source = get(obj, path);
            if (source && typeof source === 'object') {
                return omit(assign(obj, get(obj, path)), path);
            }
            else {
                return obj;
            }
        }, src),
        mapValues,
        at,
        toPairs,
        invert,
        invertBy,
        keys,
        values,
    },
    number: {
        lt,
        lte,
        gt,
        gte,
        eq,
    },
    string: {
        startsWith,
        endsWith,
        match: (src, args) => {
            return src.match(new RegExp(args.match, args.flags));
        },
        isMatch: (src, args) => {
            return (new RegExp(args.match, args.flags)).test(src);
        },
    },
    '*': {
        dive: (src, args) => (obj, key) => {
            unset(obj, key);
            set(obj, args, src);
        },
    },
};
const opToExpectedType = ((trans) => {
    let result = {};
    for (const type in trans) {
        if (trans.hasOwnProperty(type)) {
            const names = trans[type];
            for (const name in names) {
                if (names.hasOwnProperty(name)) {
                    result[name] = type;
                }
            }
        }
    }
    return result;
})(transformations);
function applyTransformations(object, args) {
    if (!args || object === null || object === undefined) {
        return object;
    }
    for (const op in args) {
        if (args.hasOwnProperty(op)) {
            const arg = args[op];
            if (op === 'and') {
                object = every(arg, predicateArgs => applyTransformations(object, predicateArgs));
                continue;
            }
            if (op === 'or') {
                object = some(arg, predicateArgs => applyTransformations(object, predicateArgs));
                continue;
            }
            const expectedType = opToExpectedType[op];
            if (!(object === undefined || object === null)) {
                let type = object && object.constructor && object.constructor.name;
                if (!type) {
                    type = typeof object;
                }
                else {
                    type = type.toLowerCase();
                }
                if (expectedType !== '*' && expectedType !== type && type !== undefined) {
                    throw Error(`"${op}" transformation expect "${expectedType}" but got "${type}"`);
                }
            }
            object = transformations[expectedType][op](object, arg);
        }
    }
    return object;
}
exports.applyTransformations = applyTransformations;
//# sourceMappingURL=transformations.js.map